package com.deviousbard.ssvcs;

import javax.swing.table.*;


public class ActionComboBoxTableModel extends AbstractTableModel {

    public static final int ACTION_COLUMN = 0;
    public static final int FILE_NAME = 1;
    public static final int LOCAL_FILE_COLUMN = 2;
    public static final int WRITABLE_COLUMN = 3;
    public static final int RE_CHECKOUT_COLUMN = 4;
    public static final int COMMENT = 5;

    protected Object[][] data;

    public ActionComboBoxTableModel() {
        Object[][] items = {{validStates[0][0], "File Name", validStates[2][0], validStates[3][0], validStates[4][0], "Comment"}};
        this.data = items;
    }

    public int getRowCount() {
        return data.length;
    }

    public int getColumnCount() {
        return COLUMN_COUNT;
    }

    public Object getValueAt(int row, int column) {
        return data[row][column];
    }

    public Class getColumnClass(int column) {
        return (data[0][column]).getClass();
    }

    public String getColumnName(int column) {
        return columnNames[column];
    }

    public boolean isCellEditable(int row, int column) {
        return (column != FILE_NAME);
    }

    public void setValueAt(Object value, int row, int column) {
        if  (isValidValue(value, column)) {
            data[row][column] = value;
            fireTableRowsUpdated(row, row);
        }
    }

    public void setData(Object[][] actions) {
        this.buildData(actions);
    }

    public static String[] getValidStates(int column, int status) {
        String [] potentialStates = validStates[column];
        if (column == ACTION_COLUMN) {
            if (SourceSafeVCSStatus.isNewFile(status)) {
                String[] validValues = {validStates[ACTION_COLUMN][0]};
                return validValues;
            } else if ((SourceSafeVCSStatus.isAvailable(status) || SourceSafeVCSStatus.isCheckedOutByAnotherUser(status)) && ! SourceSafeVCSStatus.isCheckedOutByThisUser(status)) {
                String[] validValues = {validStates[ACTION_COLUMN][1], validStates[ACTION_COLUMN][4]};
                return validValues;
            } else if (SourceSafeVCSStatus.isCheckedOutByThisUser(status)) {
                String[] validValues = {validStates[ACTION_COLUMN][2], validStates[ACTION_COLUMN][3], validStates[ACTION_COLUMN][4]};
                return validValues;
            }
        }

        if (column == LOCAL_FILE_COLUMN) {
        }

        return validStates[column];
    }

    public boolean isValidValue(Object value, int column) {
        if (value instanceof String) {
            String sValue = (String)value;

            for (int i=0; i < validStates[column].length; i++) {
                if (sValue.equals(validStates[column][i])) {
                    return true;
                }
            }
        }
        return false;
    }

    private void buildData(Object[][] actions) {
        data = new Object[actions.length / 2][6];
        for (int i=0; i < actions.length / 2; i++) {
            String fileName = (String)actions[i][0];
            int status = Integer.valueOf((String)actions[i][1]).intValue();
            data[i][0] = this.getValidStates(ACTION_COLUMN, status);
            data[i][1] = fileName;
            data[i][2] = this.getValidStates(LOCAL_FILE_COLUMN, status);
            data[i][3] = this.getValidStates(WRITABLE_COLUMN, status);
            data[i][4] = this.getValidStates(RE_CHECKOUT_COLUMN, status);
            data[i][5] = "";
        }
    }

    protected static final int COLUMN_COUNT = 6;

    protected static final String[][] validStates =
        {
            {"Add", "Checkout", "Checkin", "Undo Checkout", "Get"},
            {null},
            {"Replace", "Leave", "Delete"},
            {"No", "Yes"},
            {"No", "Yes"},
            {null}
        };

    protected static final String[] columnNames =
        {
            "Action",
            "File Name",
            "Local File",
            "Writable",
            "Re-Checkout",
            "Comment"
        };
}