package com.deviousbard.ssvcs;

import javax.swing.tree.DefaultMutableTreeNode;
import java.io.*;
import java.util.*;

public class DirectoryNode extends DefaultMutableTreeNode {
    
    File directory = null;
    boolean areChildrenDefined = false;

    public DirectoryNode(File directory) {
        this.directory = directory;
    }

    public DirectoryNode() {
        directory = new File("/");
        File[] roots = directory.listRoots();
        areChildrenDefined = true;
        TreeSet set = new TreeSet(new FileNameComparator());
        for (int i=0; i < roots.length; i++) {
            set.add(roots[i]);
        }
        for (Iterator j=set.iterator(); j.hasNext();) {
            File root = (File)j.next();
            DirectoryNode rootNode = new DirectoryNode(root);
            add(rootNode);
        }
    }
    
    public boolean isLeaf() {
        return false;
    }

    public int getChildCount() {
        if (!areChildrenDefined)
            defineChildDirectories();
        return(super.getChildCount());
    }

    private void defineChildDirectories() {
        areChildrenDefined = true;
        File[] dirList = directory.listFiles();
        TreeSet set = new TreeSet(new FileNameComparator());
        for (int i=0; i < dirList.length; i++) {
            if (dirList[i].isDirectory()) {
                set.add(dirList[i]);
            }
        }
        for (Iterator j=set.iterator(); j.hasNext();) {
            File dir = (File)j.next();
            DirectoryNode childDirNode = new DirectoryNode(dir);
            add(childDirNode);
        }
    }
    
    public String toString() {
        if (directory.getParent() == null)
            return directory.toString();
        return directory.getName();
    }
    
    public String getAbsolutePath() {
        return directory.getAbsolutePath();
    }
    
    class FileNameComparator implements Comparator {
        public int compare(Object o1, Object o2) {
            File f1 = (File)o1;
            File f2 = (File)o2;
            String fileName1 = null;
            String fileName2 = null;
            
            if (f1.getParent() == null) 
                fileName1 = f1.toString();
            else
                fileName1 = f1.getName();
                
            if (f2.getParent() == null) 
                fileName2 = f2.toString();
            else
                fileName2 = f2.getName();
                
            return fileName1.compareToIgnoreCase(fileName2);
        }
    }
}
