package com.deviousbard.ssvcs;

import javax.swing.table.*;
import javax.swing.*;
import java.util.*;
import com.borland.primetime.node.*;
import com.borland.primetime.teamdev.vcs.*;

public class MultiFileActionTableModel extends AbstractTableModel {

    public static final int ACTION_COLUMN = 0;
    public static final int FILE_NAME_COLUMN = 1;
    public static final int LOCAL_FILE_COLUMN = 2;
    public static final int WRITABLE_COLUMN = 3;
    public static final int RE_CHECKOUT_COLUMN = 4;
    public static final int COMMENT_COLUMN = 5;

    public static final ImageIcon ICON_ADD = new ImageIcon(ClassLoader.getSystemResource("com/deviousbard/ssvcs/add.gif"));
    public static final ImageIcon ICON_GET = new ImageIcon(ClassLoader.getSystemResource("com/deviousbard/ssvcs/get.gif"));
    public static final ImageIcon ICON_CHECKIN = new ImageIcon(ClassLoader.getSystemResource("com/deviousbard/ssvcs/checkin.gif"));
    public static final ImageIcon ICON_CHECKOUT = new ImageIcon(ClassLoader.getSystemResource("com/deviousbard/ssvcs/checkout.gif"));
    public static final ImageIcon ICON_UNDO_CHECKOUT = new ImageIcon(ClassLoader.getSystemResource("com/deviousbard/ssvcs/undochko.gif"));
    public static final ImageIcon ICON_NO_ACTION = new ImageIcon(ClassLoader.getSystemResource("com/deviousbard/ssvcs/noaction.gif"));

    protected Object[][] data;
    protected Node[] nodes;
    protected static int[] statuses;
    protected boolean[] isWritableStatusEnabled;
    protected boolean[] isReCheckoutStatusEnabled;

    protected static final int COLUMN_COUNT = 6;
    protected static final String[] columnNames =
        {
            "Action",
            "File Name",
            "Local File",
            "Writable",
            "Re-Checkout",
            "Comment"
        };
    public MultiFileActionTableModel() {
    }

    public MultiFileActionTableModel(Node[] nodes, int[] statuses) {
        this.nodes = nodes;
        this.statuses = statuses;
        this.isWritableStatusEnabled = new boolean[this.getRowCount()];
        this.isReCheckoutStatusEnabled = new boolean[this.getRowCount()];
        this.buildTable();
    }

    public int getRowCount() {
        return (nodes == null ? 0 : nodes.length);
    }

    public int getColumnCount() {
        return COLUMN_COUNT;
    }

    public Object getValueAt(int row, int column) {
        if (column == FILE_NAME_COLUMN) {
            if (data[row][column] instanceof com.borland.primetime.node.FileNode) {
                return ((FileNode)data[row][column]).getDisplayName();
            }
        }
        return data[row][column];
    }

    public Class getColumnClass(int column) {
        try {
            if (column == ACTION_COLUMN)
                return Class.forName("com.deviousbard.ssvcs.IconText");
            if (column == FILE_NAME_COLUMN || column == COMMENT_COLUMN || column == LOCAL_FILE_COLUMN)
                return Class.forName("java.lang.String");
            if (column == WRITABLE_COLUMN || column == RE_CHECKOUT_COLUMN)
                return Class.forName("java.lang.Boolean");
        } catch (Exception e) {
            // intentionally ignored
        }
        return null;
    }

    public String getColumnName(int column) {
        return columnNames[column];
    }

    public boolean isCellEditable(int row, int column) {

        if (column == WRITABLE_COLUMN) {
            if (isWritableStatusEnabled(((IconText)getValueAt(row, ACTION_COLUMN)).getText()))
                return true;
            else
                return false;
        }

        if (column == RE_CHECKOUT_COLUMN) {
            if (isReCheckoutStatusEnabled(((IconText)getValueAt(row, ACTION_COLUMN)).getText()))
                return true;
            else
                return false;
        }

        if (column == COMMENT_COLUMN) {
            String selectedAction = ((IconText)getValueAt(row, ACTION_COLUMN)).getText();
            if (selectedAction.equalsIgnoreCase("undo checkout") || selectedAction.equalsIgnoreCase("get") || selectedAction.equalsIgnoreCase("no action"))
                return false;
            else
                return true;
        }

        return (column != FILE_NAME_COLUMN);
    }

    public void setValueAt(Object value, int row, int column) {
        if (column == ACTION_COLUMN) {
            if (value != null) {
                String text = (String)value;
                data[row][column] = new IconText(text, getIcon(text));
            }
            else {
                data[row][column] = new IconText();
            }
            String selectedAction = ((IconText)data[row][column]).getText();
            if (selectedAction.equalsIgnoreCase("undo checkout"))
                setValueAt("Replace", row, MultiFileActionTableModel.LOCAL_FILE_COLUMN);
            else
                setValueAt("", row, MultiFileActionTableModel.LOCAL_FILE_COLUMN);
            setValueAt(new Boolean(false), row, MultiFileActionTableModel.WRITABLE_COLUMN);
            setValueAt(new Boolean(false), row, MultiFileActionTableModel.RE_CHECKOUT_COLUMN);
        }
        else {
            data[row][column] = value;
        }
        fireTableRowsUpdated(row, row);
    }

    public static String[] getValidActions(int status) {
        if (SourceSafeVCSStatus.isNewFile(status)) {
            String[] validValues = {"Add", "No Action"};
            return validValues;
        } else if ((SourceSafeVCSStatus.isAvailable(status) || SourceSafeVCSStatus.isCheckedOutByAnotherUser(status)) && ! SourceSafeVCSStatus.isCheckedOutByThisUser(status)) {
            String[] validValues = {"Checkout", "Get", "No Action"};
            return validValues;
        } else if (SourceSafeVCSStatus.isCheckedOutByThisUser(status)) {
            String[] validValues = {"Checkin", "Undo Checkout", "Get", "No Action"};
            return validValues;
        }
        return null;
    }

    public List getData() {
        SourceSafeVCS vcs = (SourceSafeVCS)VCSFactory.getVCS("SourceSafeVCS");
        ArrayList rows = new ArrayList();
        try {
            for (int i=0; i < this.getRowCount(); i++) {
                String action = ((IconText)data[i][ACTION_COLUMN]).getText();
                if (! action.equalsIgnoreCase("no action")) {
                    ArrayList row = new ArrayList();
                    row.add(action);
                    row.add((Node)data[i][FILE_NAME_COLUMN]);
                    row.add((String)data[i][LOCAL_FILE_COLUMN]);
                    row.add((Boolean)data[i][WRITABLE_COLUMN]);
                    row.add((Boolean)data[i][RE_CHECKOUT_COLUMN]);
                    row.add((String)data[i][COMMENT_COLUMN]);
                    rows.add(row);
                }
            }
        } catch (Exception e) {
            e.printStackTrace(vcs.log);
        }
        return rows;

    }

    public static String[] getValidLocalFileOptions(String action, int status) {
        if ("add | checkin".indexOf(action.toLowerCase()) != -1) {
            String[] validValues = {"", "Delete"};
            return validValues;
        } else if ("checkout".indexOf(action.toLowerCase()) != -1) {
            String[] validValues = {"", "Don't Get"};
            return validValues;
        } else if ("undo checkout".indexOf(action.toLowerCase()) != -1) {
            String[] validValues = {"Replace", "Leave", "Delete"};
            return validValues;
        } else if ("get".indexOf(action.toLowerCase()) != -1) {
            String[] validValues = {""};
            return validValues;
        } else if ("no action".indexOf(action.toLowerCase()) != -1) {
            String[] validValues = {""};
            return validValues;
        }
        return null;
    }

    public static boolean isWritableStatusEnabled(String action) {
        if ("get".indexOf(action.toLowerCase()) != -1) {
            return true;
        }
        return false;
    }

    public static boolean isReCheckoutStatusEnabled(String action) {
        if ("add | checkin".indexOf(action.toLowerCase()) != -1) {
            return true;
        }
        return false;
    }

    public boolean isValidValue(Object value, int column) {
        return true;
    }

    public boolean isReCheckoutStatusEnabled(int row) {
        return isReCheckoutStatusEnabled[row];
    }

    public boolean isWritableStatusEnabled(int row) {
        return isWritableStatusEnabled[row];
    }

    public int getStatus(int row) {
        return statuses[row];
    }

    private void buildTable() {
        data = new Object[this.getRowCount()][this.getColumnCount()];
        for (int i=0; i < nodes.length; i++) {
            String setAction = getValidActions(statuses[i])[0];
            data[i][0] = new IconText(setAction, getIcon(setAction));
            data[i][1] = nodes[i];
            data[i][2] = getValidLocalFileOptions(setAction, statuses[i])[0];
            data[i][3] = Boolean.valueOf("false");
            isWritableStatusEnabled[i] = isWritableStatusEnabled(setAction);
            data[i][4] = Boolean.valueOf("false");
            isReCheckoutStatusEnabled[i] = isReCheckoutStatusEnabled(setAction);
            data[i][5] = "";
        }
    }

    private ImageIcon getIcon(String text) {
        if (text.equalsIgnoreCase("add"))
            return ICON_ADD;
        if (text.equalsIgnoreCase("checkin"))
            return ICON_CHECKIN;
        if (text.equalsIgnoreCase("checkout"))
            return ICON_CHECKOUT;
        if (text.equalsIgnoreCase("undo checkout"))
            return ICON_UNDO_CHECKOUT;
        if (text.equalsIgnoreCase("get"))
            return ICON_GET;
        if (text.equalsIgnoreCase("no action"))
            return ICON_NO_ACTION;
        return new ImageIcon();
    }

    public static String getStatusString(int row) {
        int status = statuses[row];
        StringBuffer statusBuffer = new StringBuffer("Status: ");
        boolean statusAdded = false;
        if (SourceSafeVCSStatus.isNewFile(status)) {
            statusBuffer.append("New");
            statusAdded = true;
        }
        if (SourceSafeVCSStatus.isCheckedOutByThisUser(status)) {
            if (statusAdded) {
                statusBuffer.append(", ");
            }
            statusBuffer.append("Checked out by you");
            statusAdded = true;
        }

        if (SourceSafeVCSStatus.isCheckedOutByAnotherUser(status)) {
            if (statusAdded) {
                statusBuffer.append(", ");
            }
            statusBuffer.append("Checked out by another user");
            statusAdded = true;
        }

        if (SourceSafeVCSStatus.isAvailable(status)) {
            if (statusAdded) {
                statusBuffer.append(", ");
            }
            statusBuffer.append("Not checked out by anyone");
            statusAdded = true;
        }

        return statusBuffer.toString();
    }

    public void setData(Node[] nodes, int[] statuses) {
        this.nodes = nodes;
        this.statuses = statuses;
        this.isWritableStatusEnabled = new boolean[this.getRowCount()];
        this.isReCheckoutStatusEnabled = new boolean[this.getRowCount()];
        this.buildTable();
        this.fireTableDataChanged();
    }
}
