package com.deviousbard.ssvcs;

import javax.swing.tree.*;
import com.borland.primetime.teamdev.vcs.*;
import com.borland.primetime.util.runner.*;
import java.util.*;
import com.borland.jbuilder.node.JBProject;
import com.borland.jbuilder.*;
import com.borland.primetime.node.*;
import com.borland.primetime.*;
import com.borland.primetime.ide.*;
import com.deviousbard.opentool.util.*;

public class SourceSafeProjectNode extends DefaultMutableTreeNode {

    private String projectName = null;
    private String sourceSafeExecutablePath = null;
    private String userId = null;
    private String password = null;
    private boolean authRequired = false;
    private boolean areChildrenDefined = false;
    private static final String ROOT_PROJECT = "/";
    private SourceSafeVCS vcs = (SourceSafeVCS)VCSFactory.getVCS("SourceSafeVCS");

    public SourceSafeProjectNode(String projectName, String sourceSafeExecutablePath, String userId, String password, boolean authRequired) {
        this.projectName = projectName;
        this.sourceSafeExecutablePath = sourceSafeExecutablePath;
        this.userId = userId;
        this.password = password;
        this.authRequired = authRequired;
    }

    public boolean isLeaf() {
        return false;
    }

    public int getChildCount() {
        if (!areChildrenDefined)
            defineChildProjects();
        return(super.getChildCount());
    }

    private void defineChildProjects() {
        String auth = "";
        if (this.authRequired) {
            auth = " -Y" + this.userId + "," + this.password + " ";
        }
        areChildrenDefined = true;
        if (projectName.equals(ROOT_PROJECT))
            projectName = "$/";
        String command = "\"" + sourceSafeExecutablePath + "\"" + " Dir -F- -R- " + "\"$" + getParentProject() + "/" + projectName + "\"" + " -I-Y " + auth;
        ModifiedShellRunner shellrunner = new ModifiedShellRunner();
        shellrunner.run(command);
        if (shellrunner.getStderr().size() > 0) {
            for (Iterator iterator = shellrunner.getStderr().iterator(); iterator.hasNext();) {
                String error = (String)iterator.next();
                System.out.println("Error: " + error);
            }
            return;
        }
        int index = 0;
        for (Iterator iterator = shellrunner.getStdout().iterator(); iterator.hasNext();) {
            try {
                String childProject = (String)iterator.next();
                if (index > 0 && childProject.substring(0,1).equals("$")) {
                    childProject = childProject.substring(1);
                    SourceSafeProjectNode childProjectNode = new SourceSafeProjectNode(childProject, sourceSafeExecutablePath, this.userId, this.password, this.authRequired);
                    add(childProjectNode);
                }
                index++;
            } catch (StringIndexOutOfBoundsException soobe) {
                // Intentionally ignored because there are no more valid entries
            }
        }
    }

    public String toString() {
        return (projectName.equals("") ? "/" : projectName);
    }

    public String getParentProject() {
        SourceSafeProjectNode parent = (SourceSafeProjectNode)getParent();
        if (parent == null || parent.equals("/"))
            return "";
        return  (parent.getParentProject().equals("") ? "" : parent.getParentProject()) + (parent.projectName.equals("") ? "" : "/" + parent.projectName);
    }

    public String getAbsoluteProject() {
        return "$" + getParentProject() + "/" + projectName;
    }

    public void createProject(String parentProject, String newSubProject, String comment) {
        SourceSafeVCS vcs = (SourceSafeVCS)VCSFactory.getVCS("SourceSafeVCS");
        String auth = "";
        if (this.authRequired) {
            auth = " -Y" + this.userId + "," + this.password + " ";
        }
        String command = "\"" + sourceSafeExecutablePath + "\"" + " Create " + "\"" + parentProject + "/" + newSubProject + "\"" + " -I-Y " + auth + ((comment == null || comment.equals("")) ? "" : " -C\"" + comment + "\"");
        SourceSafeVCSUtils.runCommand(command, "Create Project", false);
    }
}
