package com.deviousbard.ssvcs;

import com.borland.primetime.teamdev.vcs.*;
import com.borland.primetime.teamdev.frontend.*;
import com.borland.primetime.util.runner.*;
import com.borland.primetime.properties.PropertyPage;
import com.borland.primetime.node.*;
import com.borland.primetime.ide.*;
import com.borland.primetime.vfs.Url;
import com.borland.primetime.actions.ActionGroup;
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.ui.*;
import java.util.*;
import javax.swing.*;
import java.io.*;
import com.borland.jbcl.layout.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.*;
import com.borland.jbuilder.*;
import com.borland.primetime.*;
import com.deviousbard.opentool.util.*;
import java.text.*;

public class SourceSafeVCS extends VCS
{
  public static final String SS_EXE = "SourceSafeVCS_exe";
  public static final String SS_MAPPINGS = "SourceSafeVCS_mappings";
  public static final String SS_USER = "SourceSafeVCS_user";
  public static final String SS_PASSWORD = "SourceSafeVCS_password";
  public static final String SS_AUTH_REQ = "SourceSafeVCS_authreq";
  public static final String SS_COMMAND_OUTPUT = "SourceSafeVCS_commandOutput";
  public static final String SS_HIST_REV = "SourceSafeVCS_histRev";

  SourceSafeVCSLogWriter log = null;

  GetFileDialog getFileDialog = null;
  CheckoutFileDialog checkoutFileDialog = null;
  CheckinFileDialog checkinFileDialog = null;
  UndoCheckoutFileDialog undoCheckoutFileDialog = null;
  AddFileDialog addFileDialog = null;
  MultiFileActionDialog multiFileActionDialog = null;
  ProjectListDialog projectListDialog = null;

  SimpleDateFormat df = new SimpleDateFormat("MM/dd/yy hh:mma");

  private Node[] nodes = null;
  private String msg = null;

  public SourceSafeVCS() {
    try {
        log = new SourceSafeVCSLogWriter();
    }
    catch (IOException ioe) {
    }

    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Register the SourceSafeVCS with the VCS Factory. This will make the class available
   * in the Team drop-down list.
   */
  public static void initOpenTool( byte major, byte minor ) {
    // VCS support was added starting from JBuilder 4
    if ( major < 4) {
      return;
    }
    VCSFactory.addVCS(new SourceSafeVCS());
  }

  /**
   * Return the property page to display when the user clicks on the Team tab.
   */

  public PropertyPage getProjectConfigPage(JBProject project) {
    return( new ConfigPage(project) );
  }

  public boolean isUnderVCS(Url url) {
    JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
    String auth="";
    String ssExe = prj.getAutoProperty(VCS.CATEGORY, SS_EXE);
    String ssProjectFile = this.getSourceSafeProjectFile(url.getFileObject(), prj);
    String ssUser = prj.getAutoProperty(VCS.CATEGORY, SS_USER);
    String ssPass = prj.getAutoProperty(VCS.CATEGORY, SS_PASSWORD);
    String ssAuthRequired = prj.getAutoProperty(VCS.CATEGORY, SS_AUTH_REQ);
    if (ssAuthRequired.equals("TRUE")) {
        auth = " -Y" + ssUser + "," + ssPass + " ";
    }
    ModifiedShellRunner shellrunner = new ModifiedShellRunner();
    String command = "\"" + ssExe + "\"" + " Status " + "\"" + ssProjectFile + "\" -I-Y " + auth;
    if (prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_COMMAND_OUTPUT).equals("TRUE"))
        shellrunner.enableMessageViewOutput(true);
    shellrunner.run(command);
    if (prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_COMMAND_OUTPUT).equals("TRUE"))
        shellrunner.enableMessageViewOutput(false);
    if (shellrunner.getStderr().size() > 0)
        return false;
    for (Iterator iterator = shellrunner.getStdout().iterator(); iterator.hasNext();) {
        String message = (String)iterator.next();
        if (message.indexOf("is not an existing filename or project") > 0)
            return false;
    }
    return true;
  }

  public byte[] getSource(Url url, RevisionInfo revInfo) {
    SourceSafeRevisionInfo ssRevInfo = (SourceSafeRevisionInfo)revInfo;
    JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
    String ssExe = prj.getAutoProperty(VCS.CATEGORY, SS_EXE);
    String ssUser = prj.getAutoProperty(VCS.CATEGORY, SS_USER);
    String ssPass = prj.getAutoProperty(VCS.CATEGORY, SS_PASSWORD);
    String ssAuthRequired = prj.getAutoProperty(VCS.CATEGORY, SS_AUTH_REQ);
    String auth="";
    if (ssAuthRequired.equals("TRUE")) {
        auth = " -Y" + ssUser + "," + ssPass + " ";
    }
    File tempFile = null;
    try {
        tempFile = File.createTempFile("SourceSafeVCS", null);
    }
    catch (IOException ioe) {
    }
    String path = tempFile.getParent();
    tempFile.delete();
    String fileName = url.getFile().substring(url.getFile().lastIndexOf('/') + 1);
    String mappedPath = this.getMappedPath(url.getFileObject(), prj);
    String fileDir = url.getFileObject().getParent();
    String filePath = path + "\\" + fileName;
    int commandCount = 3;
    String[] commands = new String[commandCount];
    String drive = path.substring(0,2);
    commands[0] = drive;
    commands[1] = "cd " + "\"" + path.substring(2) + "\"";
    commands[2] = "\"" + ssExe + "\"" + " Get " + "\"" + ssRevInfo.getProjectFile() + "\" -I-Y -GWR -V" + ssRevInfo.getSourceSafeRevision() + auth;
    ModifiedShellRunner shellrunner = new ModifiedShellRunner();
    if (prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_COMMAND_OUTPUT).equals("TRUE"))
        shellrunner.enableMessageViewOutput(true);
    shellrunner.run(commands);
    if (prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_COMMAND_OUTPUT).equals("TRUE"))
        shellrunner.enableMessageViewOutput(false);
    tempFile = new File(filePath);
    BufferedReader reader = null;
    String line = null;
    StringBuffer sourceFile = new StringBuffer();
    try {
        reader = new BufferedReader(new FileReader(tempFile));
        while ((line = reader.readLine()) != null) {
            sourceFile.append(line);
            sourceFile.append("\n");
        }
        reader.close();
        tempFile.delete();
    }
    catch (FileNotFoundException fnfe) {
        System.out.println("Could not find file: " + fnfe);
    }
    catch (IOException ioe) {
        System.out.println("Could not read File: " + ioe);
    }
    return sourceFile.toString().getBytes();
  }

  public java.util.Map getProjectStatus(Project proj) {
    JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
    FileScanner scanner = new FileScanner((JBProject)proj);
    scanner.scan();
    return null; //scanner.getFileChanges();
  }

  public VCSBrowserContextActionProvider getVCSBrowserContextActionProvider() {
    return null;
  }

  public boolean isBinary(Url url) {
    String extensions = ".gif|.jpg|.jpeg|.jar|.zip|.au|.wav|.midi";
    return extensions.indexOf(url.getFileExtension()) != -1;
  }

  public String getName() {
    return "SourceSafeVCS";
  }

  public String getDescription() {
    return "A sample VCS implementation";
  }

  public Icon getVCSIcon() {
    return BrowserIcons.ICON_FOLDER;
  }

  public ActionGroup getVCSProjectMenuGroup() {
    JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
    String ssExe = prj.getAutoProperty(VCS.CATEGORY, SS_EXE);
    String ssUser = prj.getAutoProperty(VCS.CATEGORY, SS_USER);
    if (ssExe == null || ssExe.equals("") || ssUser == null || ssUser.equals("")) {
        return null;
    }
    String ssPass = prj.getAutoProperty(VCS.CATEGORY, SS_PASSWORD);
    String ssAuthRequired = prj.getAutoProperty(VCS.CATEGORY, SS_AUTH_REQ);
    if (ssAuthRequired.equals("TRUE")) {
        if (ssPass == null || ssPass.equals("")) {
            return null;
        }
    }
    ActionGroup group = new ActionGroup();
    group.add(Actions.CREATE_PROJECT);
    group.add(Actions.GET_PROJECT);
    return group;
  }

  public ActionGroup getVCSFileMenuGroup() {
    JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
    String ssExe = prj.getAutoProperty(VCS.CATEGORY, SS_EXE);
    String ssUser = prj.getAutoProperty(VCS.CATEGORY, SS_USER);
    if (ssExe == null || ssExe.equals("") || ssUser == null || ssUser.equals("")) {
        return null;
    }
    String ssPass = prj.getAutoProperty(VCS.CATEGORY, SS_PASSWORD);
    String ssAuthRequired = prj.getAutoProperty(VCS.CATEGORY, SS_AUTH_REQ);
    if (ssAuthRequired.equals("TRUE")) {
        if (ssPass == null || ssPass.equals("")) {
            return null;
        }
    }
    ActionGroup aGroup = new ActionGroup();
    aGroup.add(Actions.ADD_FILE);
    aGroup.add(Actions.CHECKIN_FILE);
    aGroup.add(Actions.CHECKOUT_FILE);
    aGroup.add(Actions.UNDO_CHECKOUT_FILE);
    aGroup.add(Actions.GET_FILE);
    aGroup.add(Actions.STATUS_FILE);
    return(aGroup);
  }

  
  /**
   * Always return true
   */
    public boolean isConfigureVCSMenuEnabled() {
        return true;
    }

  /**
   * Return the list of actions that will be presented in the context menu in the
   * project pane.
   */
  public ActionGroup getVCSContextMenuGroup() {
    Icon icon = new ImageIcon(this.getClass().getResource("srcsafe.gif"));
    ActionGroup aGroup = new ActionGroup("Microsoft Source Safe",'S',"Manage file with Microsoft Source Safe", icon, true );
    JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
    String ssExe = prj.getAutoProperty(VCS.CATEGORY, SS_EXE);
    String ssUser = prj.getAutoProperty(VCS.CATEGORY, SS_USER);
    if (ssExe == null || ssExe.equals("") || ssUser == null || ssUser.equals("")) {
        return (aGroup);
    }
    String ssPass = prj.getAutoProperty(VCS.CATEGORY, SS_PASSWORD);
    String ssAuthRequired = prj.getAutoProperty(VCS.CATEGORY, SS_AUTH_REQ);
    if (ssAuthRequired.equals("TRUE")) {
        if (ssPass == null || ssPass.equals("")) {
            return(aGroup);
        }
    }
    aGroup.add(Actions.ADD);
    aGroup.add(Actions.CHECKIN);
    aGroup.add(Actions.CHECKOUT);
    aGroup.add(Actions.UNDO_CHECKOUT);
    aGroup.add(Actions.GET);
    aGroup.add(Actions.STATUS);
    aGroup.add(Actions.MULTI_FILE);
    return(aGroup);
  }

  public DeleteDialogInterface getDeleteDialog() {
    return null;
  }


  public void checkin( Node[] nodes, String msg ) {
    this.nodes = nodes;
    this.msg = msg;

    checkinFileDialog.reset();
    checkinFileDialog.setVisible(true);
  }

  public void checkout( Node[] nodes, String msg ) {
    this.nodes = nodes;
    this.msg = msg;

    checkoutFileDialog.reset();
    checkoutFileDialog.setVisible(true);
  }

  public void undoCheckout( Node[] nodes, String msg ) {
    this.nodes = nodes;
    this.msg = msg;

    undoCheckoutFileDialog.reset();
    undoCheckoutFileDialog.setVisible(true);
  }

  public void add( Node[] nodes, String msg ) {
    this.nodes = nodes;
    this.msg = msg;

    addFileDialog.reset();
    addFileDialog.setVisible(true);
  }

  public void get( Node[] nodes, String msg ) {
    this.nodes = nodes;
    this.msg = msg;

    getFileDialog.reset();
    getFileDialog.setVisible(true);
  }

  public void createProject(String msg ) {
    this.msg = msg;
    if (projectListDialog == null) {
        JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
        String ssExe = prj.getAutoProperty(VCS.CATEGORY, SS_EXE);
        String ssUser = prj.getAutoProperty(VCS.CATEGORY, SS_USER);
        String ssPass = prj.getAutoProperty(VCS.CATEGORY, SS_PASSWORD);
        String ssAuthRequired = prj.getAutoProperty(VCS.CATEGORY, SS_AUTH_REQ);
        projectListDialog = new ProjectListDialog(ssExe, ssUser, ssPass, ((ssAuthRequired == null || ssAuthRequired.equals("") || ssAuthRequired.equals("FALSE")) ? false : true), new Frame(), "Create Project", true);
        projectListDialog.setSize(365, 575);
        projectListDialog.btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                projectListDialog_btnCancel_actionPerformed(e);
            }
        });
    } else {
        projectListDialog.refreshSourceSafeProjectView();
    }
    projectListDialog.setVisible(true);
  }

  public void getProject(String msg ) {
    this.msg = msg;
    if (projectListDialog == null) {
        JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
        String ssExe = prj.getAutoProperty(VCS.CATEGORY, SS_EXE);
        String ssUser = prj.getAutoProperty(VCS.CATEGORY, SS_USER);
        String ssPass = prj.getAutoProperty(VCS.CATEGORY, SS_PASSWORD);
        String ssAuthRequired = prj.getAutoProperty(VCS.CATEGORY, SS_AUTH_REQ);
        projectListDialog = new ProjectListDialog(ssExe, ssUser, ssPass, ((ssAuthRequired == null || ssAuthRequired.equals("") || ssAuthRequired.equals("FALSE")) ? false : true), new Frame(), "Get Project", true);
        projectListDialog.setSize(365, 575);
        projectListDialog.btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                projectListDialog_btnCancel_actionPerformed(e);
            }
        });
    } else {
        projectListDialog.refreshSourceSafeProjectView();
    }
    projectListDialog.setVisible(true);
  }

  public void status( Node[] nodes, String msg ) {
    int status = SourceSafeVCSUtils.getStatus(((FileNode)nodes[0]).getUrl().getFileObject(), (JBProject)Browser.getActiveBrowser().getActiveProject(), true);
    String workFile = ((FileNode)nodes[0]).getUrl().getFileObject().getName();
    String message = "";
    if (SourceSafeVCSStatus.isCheckedOutByThisUser(status))
        message += "\"" + workFile + "\" is checked out by you.\n\n";
    if (SourceSafeVCSStatus.isCheckedOutByAnotherUser(status))
        message += "\"" + workFile + "\" is checked out by another user.\n\n";
    if (SourceSafeVCSStatus.isAvailable(status))
        message += "\"" + workFile + "\" is not checked out by anyone.\n\n";
    if (SourceSafeVCSStatus.isNewFile(status))
        message += "\"" + workFile + "\" is a new file.\n\n";
    if (SourceSafeVCSStatus.isDifferentFromWorkFile(status))
        message += "\"" + workFile + "\" is different from the current version in Microsoft Source Safe.\n\n";
    SourceSafeVCSUtils.showMessage(message, "File Status", JOptionPane.INFORMATION_MESSAGE);
  }

  public void multiFileActions(Node[] nodes, String msg) {
    try {
        this.nodes = nodes;
        this.msg = msg;
        int[] statuses = new int[nodes.length];
        for (int i=0; i < nodes.length; i++) {
            statuses[i] = SourceSafeVCSUtils.getStatus(((FileNode)nodes[i]).getUrl().getFileObject(),(JBProject)Browser.getActiveBrowser().getActiveProject());
        }
        ((MultiFileActionTableModel)multiFileActionDialog.actionPanel.tableFileActions.getModel()).setData(nodes, statuses);
        multiFileActionDialog.setVisible(true);
    }
    catch (Exception e) {
        e.printStackTrace(log);
    }
  }

  public String getSourceSafeProjectFile(File file, JBProject prj) {
    String fileUrl = file.getAbsolutePath().replace('\\','/');
    Map directoryMap = ((ConfigPage)this.getProjectConfigPage(prj)).getDirectoryMappings();
    String ssWorkFilePath = fileUrl.substring(0, fileUrl.lastIndexOf('/'));
    String fileName = fileUrl.substring(fileUrl.lastIndexOf('/') + 1);
    String mappedPath = this.getMappedPath(file, prj);
    String ssProjectFile = null;
    if (mappedPath != null) {
        String ssProjectPath = (String)directoryMap.get(mappedPath);
        if (ssWorkFilePath.equals(mappedPath)) {
            ssProjectFile = ssProjectPath + "/" + fileName;
        } else {
            ssProjectFile = ssProjectPath + "/" + ssWorkFilePath.substring(ssWorkFilePath.indexOf(mappedPath) + mappedPath.length() + 1) + "/" + fileName;
        }
    }
    return ssProjectFile;
  }

  public String getMappedPath(File file, JBProject prj) {
    String fileUrl = file.getAbsolutePath().replace('\\','/');
    String tmpPath = fileUrl;
    Map directoryMap = ((ConfigPage)this.getProjectConfigPage(prj)).getDirectoryMappings();
    while (tmpPath.indexOf('/') != -1) {
        tmpPath = tmpPath.substring(0, tmpPath.lastIndexOf('/'));
        for (Iterator i=directoryMap.keySet().iterator(); i.hasNext();) {
            String key = (String)i.next();
            if (tmpPath.equalsIgnoreCase(key)) {
                return key;
            }
        }
    }
    return null;
  }

  public void get_btnOK_actionPerformed(ActionEvent e) {
    getFileDialog.setVisible(false);
    getFile(nodes[0], getFileDialog.cbWritable.isSelected(), true);
  }

  public void get_btnCancel_actionPerformed(ActionEvent e) {
    getFileDialog.setVisible(false);
  }

  public void checkout_btnOK_actionPerformed(ActionEvent e) {
    checkoutFileDialog.setVisible(false);
    checkoutFile(nodes[0], checkinFileDialog.taComment.getText(), checkoutFileDialog.cbRetainLocalCopy.isSelected(), true);
  }

  public void checkout_btnCancel_actionPerformed(ActionEvent e) {
    checkoutFileDialog.setVisible(false);
  }

  public void checkin_btnOK_actionPerformed(ActionEvent e) {
    checkinFileDialog.setVisible(false);
    checkinFile(nodes[0], checkinFileDialog.taComment.getText(), checkinFileDialog.cbRemoveLocalCopy.isSelected(), checkinFileDialog.cbKeepCheckedOut.isSelected(), checkinFileDialog.cbLeaveWritable.isSelected(), true);
  }

  public void checkin_btnCancel_actionPerformed(ActionEvent e) {
    checkinFileDialog.setVisible(false);
  }

  public void undoCheckout_btnOK_actionPerformed(ActionEvent e) {
    undoCheckoutFileDialog.setVisible(false);
    String selectedItem = (String)undoCheckoutFileDialog.localFileComboBoxModel.getSelectedItem();
    undoCheckoutFile(nodes[0], selectedItem, true);
  }

  public void undoCheckout_btnCancel_actionPerformed(ActionEvent e) {
    undoCheckoutFileDialog.setVisible(false);
  }

  public void add_btnOK_actionPerformed(ActionEvent e) {
    addFileDialog.setVisible(false);
    addFile(nodes[0], addFileDialog.taComment.getText(), addFileDialog.cbImmediateCheckout.isSelected(), addFileDialog.cbLeaveWritable.isSelected(), addFileDialog.cbRemoveLocalCopy.isSelected(), true);
  }

  public void add_btnCancel_actionPerformed(ActionEvent e) {
    addFileDialog.setVisible(false);
  }

  public void multiFileAction_btnOK_actionPerformed(ActionEvent e) {
    try {
        java.util.List actions = ((MultiFileActionTableModel)multiFileActionDialog.actionPanel.tableFileActions.getModel()).getData();
        StringBuffer message = new StringBuffer();
        for (Iterator i = actions.iterator(); i.hasNext();) {
            java.util.List fileAction = (java.util.List)i.next();
            Iterator j = fileAction.iterator();
            String command = (String)j.next();
            Node node = (Node)j.next();
            String localFile = (String)j.next();
            boolean writable = ((Boolean)j.next()).booleanValue();
            boolean reCheckOut = ((Boolean)j.next()).booleanValue();
            String comment = (String)j.next();
            if (command.equalsIgnoreCase("add")) {
                message.append(addFile(node, comment, reCheckOut, writable, (localFile.equalsIgnoreCase("delete") ? true : false), false) + "\n");
            } else if (command.equalsIgnoreCase("checkin")) {
                message.append(checkinFile(node, comment, (localFile.equalsIgnoreCase("delete") ? true : false), reCheckOut, writable, false) + "\n");
            } else if (command.equalsIgnoreCase("checkout")) {
                message.append(checkoutFile(node, comment, (localFile.equalsIgnoreCase("don't get") ? true : false), false) + "\n");
            } else if (command.equalsIgnoreCase("get")) {
                message.append(getFile(node, writable, false) + "\n");
            } else if (command.equalsIgnoreCase("undo checkout")) {
                message.append(undoCheckoutFile(node, localFile, false) + "\n");
            }
        }
        multiFileActionDialog.setVisible(false);
        SourceSafeVCSUtils.showMessage(message.toString(), "Multi-File Actions", JOptionPane.INFORMATION_MESSAGE);
    } catch (Exception ex) {
        ex.printStackTrace(log);
    }

  }

  public void multiFileAction_btnCancel_actionPerformed(ActionEvent e) {
    multiFileActionDialog.setVisible(false);
  }

  public void projectListDialog_btnCancel_actionPerformed(ActionEvent e) {
    projectListDialog.setVisible(false);
  }

  private void jbInit() throws Exception {
    getFileDialog = new GetFileDialog(new Frame(), "Get File", true);
    getFileDialog.setSize(230, 120);
    getFileDialog.btnOK.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
            get_btnOK_actionPerformed(e);
        }
    });
    getFileDialog.btnCancel.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
            get_btnCancel_actionPerformed(e);
        }
    });

    checkoutFileDialog = new CheckoutFileDialog(new Frame(), "Checkout File", true);
    checkoutFileDialog.setSize(245, 225);
    checkoutFileDialog.btnOK.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
            checkout_btnOK_actionPerformed(e);
        }
    });
    checkoutFileDialog.btnCancel.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
            checkout_btnCancel_actionPerformed(e);
        }
    });

    checkinFileDialog = new CheckinFileDialog(new Frame(), "Checkin File", true);
    checkinFileDialog.setSize(260, 235);
    checkinFileDialog.btnOK.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
            checkin_btnOK_actionPerformed(e);
        }
    });
    checkinFileDialog.btnCancel.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
            checkin_btnCancel_actionPerformed(e);
        }
    });

    undoCheckoutFileDialog = new UndoCheckoutFileDialog(new Frame(), "Undo Checkout File", true);
    undoCheckoutFileDialog.setSize(270, 130);
    undoCheckoutFileDialog.btnOK.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
            undoCheckout_btnOK_actionPerformed(e);
        }
    });
    undoCheckoutFileDialog.btnCancel.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
            undoCheckout_btnCancel_actionPerformed(e);
        }
    });

    addFileDialog = new AddFileDialog(new Frame(), "Add File", true);
    addFileDialog.setSize(275, 260);
    addFileDialog.btnOK.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
            add_btnOK_actionPerformed(e);
        }
    });
    addFileDialog.btnCancel.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
            add_btnCancel_actionPerformed(e);
        }
    });

    multiFileActionDialog = new MultiFileActionDialog(new Frame(), "Multiple File Actions", true);
    multiFileActionDialog.setSize(710, 330);
    multiFileActionDialog.btnOK.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
            multiFileAction_btnOK_actionPerformed(e);
        }
    });
    multiFileActionDialog.btnCancel.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
            multiFileAction_btnCancel_actionPerformed(e);
        }
    });
  }

    public Vector getRevisions(Url url) {
        Vector revs = new Vector();
        try {
            JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
            File workFile = url.getFileObject();
            String ssExe = prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_EXE);
            String ssUser = prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_USER).toLowerCase();
            String ssProjectFile = getSourceSafeProjectFile(workFile, prj);
            ModifiedShellRunner shellrunner = new ModifiedShellRunner();
            String auth = "";
            String ssPass = prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_PASSWORD);
            String ssAuthRequired = prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_AUTH_REQ);
            if (ssAuthRequired.equals("TRUE")) {
                auth = " -Y" + ssUser + "," + ssPass + " ";
            }
            String[] commands = new String[1];
            String ssHistRev = prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_HIST_REV);
            commands[0] = "\"" + ssExe + "\"" + " History " + "\"" + ssProjectFile + "\" -L-" + auth;
            if (prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_COMMAND_OUTPUT).equals("TRUE"))
                shellrunner.enableMessageViewOutput(true);
            shellrunner.run(commands);
            if (prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_COMMAND_OUTPUT).equals("TRUE"))
                shellrunner.enableMessageViewOutput(false);
            if (shellrunner.getStdout().size() > 0) {
                Iterator iterator = shellrunner.getStdout().iterator();
                // First couple of rows of output are throw away lines
                String histRow = "";
                Date changeDate = null;
                int revCount = 0;
                while (iterator.hasNext() && revCount < Integer.parseInt(ssHistRev)) {
                    String revNumber = "";
                    String who = "";
                    String date = "";
                    String time = "";
                    String label = "";
                    String desc = "";
                    while (histRow.indexOf("**") == -1) {
                        if (iterator.hasNext()) {
                            histRow = (String)iterator.next();
                        }
                    }
                    int versionIndex = histRow.indexOf("Version") + 8;
                    if (versionIndex >= 8) {
                        revNumber = histRow.substring(versionIndex, histRow.indexOf("*", versionIndex) - 1).trim();
                    }
                    if (iterator.hasNext()) {
                        histRow = (String)iterator.next();
                        int userIndex = histRow.indexOf("User:") + 6;
                        who = histRow.substring(userIndex, histRow.indexOf("Date:", userIndex) - 1).trim();
                        int dateIndex = histRow.indexOf("Date:") + 6;
                        date = histRow.substring(dateIndex, histRow.indexOf("Time:", dateIndex) - 1).trim();
                        int timeIndex = histRow.indexOf("Time:") + 6;
                        time = histRow.substring(timeIndex).trim() + "m";
                        try {
                            changeDate = df.parse(((date.length() < 8) ? "0" + date : date) + " " + ((time.length() < 7) ? "0" + time : time));
                        }
                        catch (ParseException pe) {
                            System.out.println("Parse Error: " + pe);
                            changeDate = new Date();
                        }
                    }
                    if (iterator.hasNext()) {
                        histRow = (String)iterator.next();
                        label = histRow.trim();
                    }
                    if (iterator.hasNext()) {
                        histRow = (String)iterator.next();
                        desc = null;
                        try {
                            int commentIndex = histRow.indexOf("Comment:") + 9;
                            desc = histRow.substring(commentIndex).trim();
                        }
                        catch (ArrayIndexOutOfBoundsException aioobe) {
                            desc = "";
                        }
                        catch (StringIndexOutOfBoundsException sioobe) {
                            desc = "";
                        }
                    }
                    while(iterator.hasNext() && (histRow.indexOf("**") == -1)) {
                        histRow = (String)iterator.next();
                        if (histRow.indexOf("**") == -1)
                            desc += "\n" + histRow.trim();
                    }
                    SourceSafeRevisionInfo r = new SourceSafeRevisionInfo(revNumber, changeDate.getTime(), who, desc, label);
                    r.setSourceSafeRevision(revNumber);
                    r.setProjectFile(ssProjectFile);
                    revs.add(r);
                    revCount++;
                }
            }
        }
        catch (Exception ex) {
            log.writeLogEntry("Error In History: " + ex);
            ex.printStackTrace();
        }
        return revs;
    }

    private String checkinFile(Node node, String comment, boolean removeLocalCopy, boolean keepCheckedOut, boolean leaveWritable, boolean showMessage) {
        String keepCheckedOutOption = "";
        String removeLocalCopyCommand = "";
        String leaveWritableOption = "";
        if (comment == null) {
            comment = "";
        }
        if (keepCheckedOut) {
            keepCheckedOutOption = " -K ";
        }
        if (leaveWritable) {
            leaveWritableOption = " -W ";
        }
        JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
        String ssExe = prj.getAutoProperty(VCS.CATEGORY, SS_EXE);
        File workFile;
        workFile = ((FileNode)node).getUrl().getFileObject();
        String path = workFile.getParent();
        String ssUser = prj.getAutoProperty(VCS.CATEGORY, SS_USER);
        String ssPass = prj.getAutoProperty(VCS.CATEGORY, SS_PASSWORD);
        String ssAuthRequired = prj.getAutoProperty(VCS.CATEGORY, SS_AUTH_REQ);
        String auth = "";
        if (ssAuthRequired.equals("TRUE")) {
            auth = " -Y" + ssUser + "," + ssPass + " ";
        }
        int commandCount = 3;
        if (removeLocalCopy) {
            removeLocalCopyCommand = "DEL \"" + workFile.getAbsoluteFile() + "\"";
            commandCount++;
        }
        String[] commands = new String[commandCount];
        String drive = path.substring(0,2);
        commands[0] = drive;
        commands[1] = "cd " + "\"" + path.substring(2) + "\"";
        commands[2] = "\"" + ssExe + "\"" + " Checkin " + "\"" + this.getSourceSafeProjectFile(((FileNode)node).getUrl().getFileObject(), prj) + "\" -I-Y " + keepCheckedOutOption + leaveWritableOption + auth + (comment.equals("") ? "" : " -C\"" + comment + "\" ");
        if (commandCount > 3) {
            commands[3] = removeLocalCopyCommand;
        }
        return SourceSafeVCSUtils.runCommands(commands, "Checkin File", showMessage);
    }

    private String checkoutFile(Node node, String comment, boolean retainLocalCopy, boolean showMessage) {
        String retainLocalCopyOption = "";
        if (comment == null) {
            comment = "";
        }
        if (retainLocalCopy) {
            retainLocalCopyOption = " -G- ";
        }
        JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
        String ssExe = prj.getAutoProperty(VCS.CATEGORY, SS_EXE);
        String ssUser = prj.getAutoProperty(VCS.CATEGORY, SS_USER);
        String ssPass = prj.getAutoProperty(VCS.CATEGORY, SS_PASSWORD);
        String ssAuthRequired = prj.getAutoProperty(VCS.CATEGORY, SS_AUTH_REQ);
        String auth = "";
        if (ssAuthRequired.equals("TRUE")) {
            auth = " -Y" + ssUser + "," + ssPass + " ";
        }
        File workFile;
        workFile = ((FileNode)node).getUrl().getFileObject();
        String path = workFile.getParent();
        int commandCount = 3;
        String[] commands = new String[commandCount];
        String drive = path.substring(0,2);
        commands[0] = drive;
        commands[1] = "cd " + "\"" + path.substring(2) + "\"";
        commands[2] = "\"" + ssExe + "\"" + " Checkout " + "\"" + this.getSourceSafeProjectFile(((FileNode)node).getUrl().getFileObject(), prj) + "\" -I-Y -GWR" + retainLocalCopyOption + auth + (comment.equals("") ? "" : " -C\"" + comment + "\" ");
        return SourceSafeVCSUtils.runCommands(commands, "Checkout File", showMessage);
    }

    private String getFile(Node node, boolean makeWritable, boolean showMessage) {
        String makeWritableOption = "";
        if (makeWritable) {
            makeWritableOption = " -W ";
        }

        JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
        String ssExe = prj.getAutoProperty(VCS.CATEGORY, SS_EXE);
        String ssUser = prj.getAutoProperty(VCS.CATEGORY, SS_USER);
        String ssPass = prj.getAutoProperty(VCS.CATEGORY, SS_PASSWORD);
        String ssAuthRequired = prj.getAutoProperty(VCS.CATEGORY, SS_AUTH_REQ);
        String auth="";
        if (ssAuthRequired.equals("TRUE")) {
            auth = " -Y" + ssUser + "," + ssPass + " ";
        }
        File workFile;
        workFile = ((FileNode)node).getUrl().getFileObject();
        String path = workFile.getParent();
        int commandCount = 3;
        String[] commands = new String[commandCount];
        String drive = path.substring(0,2);
        commands[0] = drive;
        commands[1] = "cd " + "\"" + path.substring(2) + "\"";
        commands[2] = "\"" + ssExe + "\"" + " Get " + "\"" + this.getSourceSafeProjectFile(((FileNode)node).getUrl().getFileObject(), prj) + "\" -I-Y -GWR" + makeWritableOption + auth;
        return SourceSafeVCSUtils.runCommands(commands, "Get File", showMessage);
    }

    private String undoCheckoutFile(Node node, String localFileDisposition, boolean showMessage) {
        JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
        String ssExe = prj.getAutoProperty(VCS.CATEGORY, SS_EXE);
        File workFile;
        workFile = ((FileNode)node).getUrl().getFileObject();
        String path = workFile.getParent();
        String localFileOption = "";
        String deleteCommand = "";
        int commandCount = 3;
        if (localFileDisposition.equals("Leave")) {
            localFileOption = " -G- ";
        }
        else if (localFileDisposition.equals("Delete")) {
            deleteCommand = "DEL \"" + workFile.getAbsoluteFile() + "\"";
            commandCount++;
        }
        String ssUser = prj.getAutoProperty(VCS.CATEGORY, SS_USER);
        String ssPass = prj.getAutoProperty(VCS.CATEGORY, SS_PASSWORD);
        String ssAuthRequired = prj.getAutoProperty(VCS.CATEGORY, SS_AUTH_REQ);
        String auth = "";
        if (ssAuthRequired.equals("TRUE")) {
            auth = " -Y" + ssUser + "," + ssPass + " ";
        }
        String[] commands = new String[commandCount];
        String drive = path.substring(0,2);
        commands[0] = drive;
        commands[1] = "cd " + "\"" + path.substring(2) + "\"";
        commands[2] = "\"" + ssExe + "\"" + " UndoCheckout " + "\"" + this.getSourceSafeProjectFile(((FileNode)node).getUrl().getFileObject(), prj) + "\" -I-Y " + localFileOption  + auth;
        if (commandCount > 3) {
            commands[3] = deleteCommand;
        }
        return SourceSafeVCSUtils.runCommands(commands, "Undo Checkout File", showMessage);
    }

    private String addFile(Node node, String comment, boolean immediateCheckout, boolean leaveWritable, boolean removeLocalCopy, boolean showMessage) {
        String immediateCheckoutOption = "";
        String leaveWritableOption = "";
        String removeLocalCopyCommand = "";
        String auth = "";
        if (comment == null) {
            comment = "";
        }
        if (immediateCheckout) {
            immediateCheckoutOption = " -K ";
        }
        if (leaveWritable) {
            leaveWritableOption = " -W ";
        }
        JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
        String ssExe = prj.getAutoProperty(VCS.CATEGORY, SS_EXE);
        File workFile;
        workFile = ((FileNode)node).getUrl().getFileObject();
        int commandCount = 3;
        String path = workFile.getParent();
        if (removeLocalCopy) {
            removeLocalCopyCommand = "DEL \"" + workFile.getAbsoluteFile() + "\"";
            commandCount++;
        }
        String ssUser = prj.getAutoProperty(VCS.CATEGORY, SS_USER);
        String ssPass = prj.getAutoProperty(VCS.CATEGORY, SS_PASSWORD);
        String ssAuthRequired = prj.getAutoProperty(VCS.CATEGORY, SS_AUTH_REQ);
        if (ssAuthRequired.equals("TRUE")) {
            auth = " -Y" + ssUser + "," + ssPass + " ";
        }
        String projectFile = this.getSourceSafeProjectFile(workFile, prj);
        String currentProject = projectFile.substring(0, projectFile.lastIndexOf("/"));
        String[] commands = new String[commandCount];
        commands[0] = "cd " + "\"" + path + "\"";
        commands[1] = "\"" + ssExe + "\"" + " CP " + "\"" + currentProject + "\"" + auth;
        commands[2] = "\"" + ssExe + "\"" + " Add " + "\"" + workFile + "\" -I-Y " + (comment.equals("") ? "" : " -C\"" + comment + "\" ") + leaveWritableOption + immediateCheckoutOption + auth;
        if (commandCount > 3) {
            commands[3] = removeLocalCopyCommand;
        }
        return SourceSafeVCSUtils.runCommands(commands, "Add File", showMessage);
    }

  private class FileScanner {
    private JBProject project = null;
    private File projectDir = null;
    private String projectParent = null;
    private java.util.List fileChanges = new ArrayList();
    private java.util.List sourcePaths = new ArrayList();
    private int pparentLen=0;

    private FileScanner( JBProject project ) {
      try {
        this.project = project;
        File projectDir = project.getProjectPath().getFileObject();
        String sourcePath = project.getProperty(JBProject.PROPERTY_SOURCEPATH);
        String projectPath = project.getProjectPath().getFile();
        StringTokenizer st = new StringTokenizer(sourcePath, ";");
        while (st.hasMoreTokens()) {
            String tmpSrcPath = st.nextToken();
            String tmpPrjPath = projectPath;
            int index = -1;
            while ((index = tmpSrcPath.indexOf("../")) > -1) {
                tmpPrjPath = tmpPrjPath.substring(0, tmpPrjPath.lastIndexOf("/") - 1);
                tmpSrcPath = tmpSrcPath.substring(index + 3);
            }
            sourcePaths.add(tmpPrjPath + "/" + tmpSrcPath);
        }
      }
      catch (Exception ex) {
        ex.printStackTrace();
      }
    }

    public java.util.List getFileChanges() {
        return fileChanges;
    }

    public void scan() {
      for (Iterator i = sourcePaths.iterator(); i.hasNext();) {
          String path = (String)i.next();
          File f = new File(path);
          scan(f);
      }
    }

    public void scan(File path) {
      try {
        File[] list = path.listFiles();
        int len = list.length;
        for ( int i=0; i<len; i++ ) {
          // Skip status files
          if ( list[i].isDirectory() ) {
            scan( list[i] );
          }
          else {

              String fname = list[i].getCanonicalPath();
              // Ignore JBuilder and Source Safe Generated files
              if ( list[i].getName().endsWith("~") || list[i].getName().endsWith("jpx.local") || list[i].getName().endsWith("scc")) {
                continue;
              }
              else {
                  int status = SourceSafeVCSUtils.getStatus(list[i], project);
                  Url url = new Url(list[i]);
                  if ( SourceSafeVCSStatus.isNewFile(status) ) {
                    // Added
                    fileChanges.add(new VCSFileInfo(url, new RevisionStatus(RevisionStatus.STATUS_NEW)));
                    continue;
                  } else if ( SourceSafeVCSStatus.isDifferentFromWorkFile(status) ) {
                    // Modified
                    fileChanges.add(new VCSFileInfo(url, new RevisionStatus(RevisionStatus.STATUS_WSP_CHANGE)));
                    continue;
                  }
//                  if ( lastRev.compareTo(workRev) == 1 ) {
//                    // Modified in the repo
//                    mods.add(new VCSFileInfo(url,new RevisionStatus(RevisionStatus.STATUS_REPO_CHANGE)));
//                    continue;
//                  }
              }
          }
        }
      }
      catch (Exception ex) {
        ex.printStackTrace();
      }
    }
  }
}
