package com.deviousbard.ssvcs;

import com.borland.primetime.util.runner.*;
import com.borland.primetime.util.*;
import com.borland.primetime.teamdev.vcs.*;
import com.borland.primetime.teamdev.frontend.VCSBrowserContextActionProvider;
import com.borland.primetime.properties.PropertyPage;
import com.borland.primetime.node.*;
import com.borland.primetime.ide.*;
import com.borland.primetime.vfs.Url;
import com.borland.primetime.actions.ActionGroup;
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.ui.*;
import java.util.*;
import javax.swing.*;
import java.io.*;
import com.borland.jbcl.layout.*;
import java.awt.*;
import com.deviousbard.opentool.util.*;


public class SourceSafeVCSUtils {

    public SourceSafeVCSUtils() {
    }

    public static int getStatus() {
        JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
        Node[] nodes = Browser.getActiveBrowser().getProjectView().getSelectedNodes();
        File workFile = ((FileNode)nodes[0]).getUrl().getFileObject();
        return getStatus(workFile, prj);
    }

    public static int checkDifferences(File workFile, String projectFile, JBProject prj) {
        String ssExe = prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_EXE);
        String path = workFile.getParent();
        ModifiedShellRunner shellrunner = new ModifiedShellRunner();
        String auth = "";
        String ssUser = prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_USER);
        String ssPass = prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_PASSWORD);
        String ssAuthRequired = prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_AUTH_REQ);
        if (ssAuthRequired.equals("TRUE")) {
            auth = " -Y" + ssUser + "," + ssPass + " ";
        }
        int commandCount = 3;
        String[] commands = new String[commandCount];
        String drive = path.substring(0,2);
        commands[0] = drive;
        commands[1] = "cd " + "\"" + path.substring(2) + "\"";
        commands[2] = "\"" + ssExe + "\"" + " Diff " + "\"" + projectFile + "\" -B" + auth;
        int status = 0;
        if (prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_COMMAND_OUTPUT).equals("TRUE"))
            shellrunner.enableMessageViewOutput(true);
        shellrunner.run(commands);
        if (prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_COMMAND_OUTPUT).equals("TRUE"))
            shellrunner.enableMessageViewOutput(false);
        if (shellrunner.getStderr().size() > 0)
            status = 0;
        for (Iterator iterator = shellrunner.getStdout().iterator(); iterator.hasNext();) {
            String message = (String)iterator.next();
            if (message.indexOf("The files are different.") != -1)
                status += SourceSafeVCSStatus.DIFFERENT_FROM_WORKFILE;
        }
        return status;
    }

    public static int getStatus(File workFile, JBProject prj) {
        return getStatus(workFile, prj, false);
    }
    
    public static int getStatus(File workFile, JBProject prj, boolean checkForDifferences) {
        SourceSafeVCS vcs = (SourceSafeVCS)VCSFactory.getVCS("SourceSafeVCS");
        String ssExe = prj.getAutoProperty(VCS.CATEGORY, vcs.SS_EXE);
        String ssUser = prj.getAutoProperty(VCS.CATEGORY, vcs.SS_USER).toLowerCase();
        String ssProjectFile = vcs.getSourceSafeProjectFile(workFile, prj);
        ModifiedShellRunner shellrunner = new ModifiedShellRunner();
        String auth = "";
        String ssPass = prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_PASSWORD);
        String ssAuthRequired = prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_AUTH_REQ);
        if (ssAuthRequired.equals("TRUE")) {
            auth = " -Y" + ssUser + "," + ssPass + " ";
        }
        String command = "\"" + ssExe + "\"" + " Status " + "\"" + ssProjectFile + "\"" + auth;
        if (prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_COMMAND_OUTPUT).equals("TRUE"))
            shellrunner.enableMessageViewOutput(true);
        shellrunner.run(command);
        if (prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_COMMAND_OUTPUT).equals("TRUE"))
            shellrunner.enableMessageViewOutput(false);
        int status = 0;
        if (shellrunner.getStderr().size() > 0)
            status += SourceSafeVCSStatus.NEW_FILE;
        for (Iterator iterator = shellrunner.getStdout().iterator(); iterator.hasNext();) {
            String message = (String)iterator.next();
            if (message.indexOf("is not an existing filename or project") != -1)
                status += SourceSafeVCSStatus.NEW_FILE;
            else if (message.indexOf("No checked out files found.") != -1)
                status += SourceSafeVCSStatus.AVAILABLE;
            else if (message.toLowerCase().indexOf(ssUser) != -1)
                status += SourceSafeVCSStatus.CHECKED_OUT_BY_THIS_USER;
            else
                if (! SourceSafeVCSStatus.isCheckedOutByAnotherUser(status)) {
                    StringTokenizer st = new StringTokenizer(message);
                    if (st.countTokens() > 1)
                        status += SourceSafeVCSStatus.CHECKED_OUT_BY_ANOTHER_USER;
                }
        }
        if (checkForDifferences) {
            status += SourceSafeVCSUtils.checkDifferences(workFile, ssProjectFile, prj);
        }
        return status;
    }

    public static void runCommand(String command, String title, boolean showMessage) {
        JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
        ModifiedShellRunner shellrunner = new ModifiedShellRunner();
        if (prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_COMMAND_OUTPUT).equals("TRUE")) {
            shellrunner.enableMessageViewOutput(true);
        }
        shellrunner.run(command, false);
        if (prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_COMMAND_OUTPUT).equals("TRUE")) {
            shellrunner.enableMessageViewOutput(false);
        }
        if (showMessage) {
            String message = "";
            String warningMessage = "";
            if (shellrunner.getStderr().size() > 0) {
                for (Iterator iterator = shellrunner.getStderr().iterator(); iterator.hasNext();) {
                    warningMessage += (String)iterator.next() + "\n";
                }
                if (shellrunner.getStdout().size() > 0) {
                    warningMessage = title + " Warning:\n\n" + warningMessage + "\n";
                }
                else {
                    warningMessage = title + " Error:\n\n" + warningMessage + "\n";
                }
            }
            String infoMessage = "";
            for (Iterator iterator = shellrunner.getStdout().iterator(); iterator.hasNext();) {
                infoMessage += (String)iterator.next() + "\n";
            }
            infoMessage = title + " Completed:\n\n" + infoMessage;
            message = warningMessage + infoMessage;
            SourceSafeVCSUtils.showMessage(message, title, (shellrunner.getStderr().size() > 0 ? (shellrunner.getStdout().size() > 0 ? JOptionPane.WARNING_MESSAGE : JOptionPane.ERROR_MESSAGE) : JOptionPane.INFORMATION_MESSAGE));
        }
    }

    public static void runCommand(String command, boolean showMessage) {
        SourceSafeVCSUtils.runCommand(command, null, showMessage);
    }

    public static void runCommand(String command, String title) {
        SourceSafeVCSUtils.runCommand(command, title, true);
    }

    public static String runCommands(String[] commands, String title, boolean showMessage) {
        JBProject prj = (JBProject)Browser.getActiveBrowser().getActiveProject();
        ModifiedShellRunner shellrunner = new ModifiedShellRunner();
        if (prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_COMMAND_OUTPUT).equals("TRUE")) {
            shellrunner.enableMessageViewOutput(true);
        }
        shellrunner.run(commands, false);
        if (prj.getAutoProperty(VCS.CATEGORY, SourceSafeVCS.SS_COMMAND_OUTPUT).equals("TRUE")) {
            shellrunner.enableMessageViewOutput(false);
        }
        String message = "";
        String warningMessage = "";
        if (shellrunner.getStderr().size() > 0) {
            for (Iterator iterator = shellrunner.getStderr().iterator(); iterator.hasNext();) {
                warningMessage += (String)iterator.next() + "\n";
            }
            if (shellrunner.getStdout().size() > 0) {
                warningMessage = title + " Warning:\n\n" + warningMessage + "\n";
            }
            else {
                warningMessage = title + " Error:\n\n" + warningMessage + "\n";
            }
        }
        String infoMessage = "";
        for (Iterator iterator = shellrunner.getStdout().iterator(); iterator.hasNext();) {
            infoMessage += (String)iterator.next() + "\n";
        }
        infoMessage = title + " Completed:\n\n" + infoMessage;
        message = warningMessage + infoMessage;
        if (showMessage) {
            SourceSafeVCSUtils.showMessage(message, title, (shellrunner.getStderr().size() > 0 ? (shellrunner.getStdout().size() > 0 ? JOptionPane.WARNING_MESSAGE : JOptionPane.ERROR_MESSAGE) : JOptionPane.INFORMATION_MESSAGE));
        }
        return message;
    }

    public static void runCommands(String[] commands, boolean showMessage) {
        SourceSafeVCSUtils.runCommands(commands, null, showMessage);
    }

    public static void runCommands(String[] commands, String title) {
        SourceSafeVCSUtils.runCommands(commands, title, true);
    }

    public static void showMessage(String message, String title, int messageType) {
        MessageLabel messagelabel = new MessageLabel(true);
        messagelabel.setText(message);
        messagelabel.setAlignmentY(MessageLabel.CENTER_ALIGNMENT);
        JScrollPane jscrollpane = new JScrollPane(messagelabel);
        jscrollpane.setPreferredSize(new Dimension(400, 125));
        JOptionPane.showMessageDialog(Browser.getActiveBrowser(), jscrollpane, title, messageType);
    }

}
